/**********************************************************************************
 * $URL: https://source.sakaiproject.org/svn/search/tags/sakai_2-4-0/search-impl/impl/src/java/org/sakaiproject/search/component/service/impl/SearchResultResponseImpl.java $
 * $Id: SearchResultResponseImpl.java 29315 2007-04-20 14:28:12Z ajpoland@iupui.edu $
 ***********************************************************************************
 *
 * Copyright (c) 2003, 2004, 2005, 2006 The Sakai Foundation.
 *
 * Licensed under the Educational Community License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.opensource.org/licenses/ecl1.php
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **********************************************************************************/

package org.sakaiproject.search.component.service.impl;

import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.highlight.Highlighter;
import org.apache.lucene.search.highlight.QueryScorer;
import org.apache.lucene.search.highlight.Scorer;
import org.sakaiproject.search.api.EntityContentProducer;
import org.sakaiproject.search.api.SearchIndexBuilder;
import org.sakaiproject.search.api.SearchResult;
import org.sakaiproject.search.api.SearchService;
import org.sakaiproject.search.api.TermFrequency;
import org.sakaiproject.search.component.Messages;
import org.xml.sax.Attributes;



/**
 * @author ieb
 */
public class SearchResultResponseImpl implements SearchResult
{

	private static Log log = LogFactory.getLog(SearchResultResponseImpl.class);

	String[] fieldNames = null;

	private Query query = null;

	private Analyzer analyzer = null;


	private SearchIndexBuilder searchIndexBuilder;

	private SearchService searchService;

	private Map attributes;

	public SearchResultResponseImpl(Map attributes, Query query,
			Analyzer analyzer, 
			SearchIndexBuilder searchIndexBuilder, SearchService searchService)
			throws IOException
	{

		this.attributes = attributes;
		this.query = query;
		this.analyzer = analyzer;
		this.searchIndexBuilder = searchIndexBuilder;
		this.searchService = searchService;
	}
	public SearchResultResponseImpl(Attributes atts, Query query,
			Analyzer analyzer, 
			SearchIndexBuilder searchIndexBuilder, SearchService searchService)
			throws IOException
	{
		Map m = new HashMap();
		for ( int i = 0; i < atts.getLength(); i++ ) {
			m.put(atts.getLocalName(i),atts.getValue(i));
		}
		try
		{
			String title = (String)m.get("title"); //$NON-NLS-1$
			if ( title != null ) {
				m.put("title", new String(Base64.decodeBase64(title.getBytes("UTF-8")),"UTF-8")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		}
		catch (UnsupportedEncodingException e)
		{
		}
		this.attributes = m;
		this.query = query;
		this.analyzer = analyzer;
		this.searchIndexBuilder = searchIndexBuilder;
		this.searchService = searchService;
	}

	public float getScore()
	{
		return Float.parseFloat((String)attributes.get("score")); //$NON-NLS-1$
	}

	public String getId()
	{
		return (String)attributes.get("sid"); //$NON-NLS-1$
	}

	public String[] getFieldNames()
	{

		if (fieldNames != null)
		{
			return fieldNames;
		}
		fieldNames = new String[attributes.size()];
		int ii = 0;
		for (Iterator i = attributes.keySet().iterator(); i.hasNext();)
		{
			fieldNames[ii++] = (String) i.next();
		}
		return fieldNames;
	}

	public String[] getValues(String fieldName)
	{
		return new String[] {(String)attributes.get(fieldName)};
	}

	/**
	 * {@inheritDoc}
	 */
	public Map getValueMap()
	{
		HashMap hm = new HashMap();
		String[] fieldNames = getFieldNames();
		for (int i = 0; i < fieldNames.length; i++)
		{
			hm.put(fieldNames[i], new String[] { (String) attributes
					.get(fieldNames[i]) });
		}
		return hm;
	}

	public String getUrl()
	{
		return (String) attributes.get("url"); //$NON-NLS-1$
	}

	public String getTitle()
	{
		return StringUtils.escapeHtml((String) attributes
				.get("title"), false); //$NON-NLS-1$
	}

	public String getTool()
	{
		return StringUtils.escapeHtml((String) attributes
				.get("tool"), false); //$NON-NLS-1$

	}

	public int getIndex()
	{
		return Integer.parseInt((String)attributes.get("index")); //$NON-NLS-1$
	}

	public String getSearchResult()
	{
		try
		{
			Scorer scorer = new QueryScorer(query);
			Highlighter hightlighter = new Highlighter(scorer);
			StringBuffer sb = new StringBuffer();
			// contents no longer contains the digested contents, so we need to
			// fetch it from the EntityContentProducer

			EntityContentProducer sep = searchIndexBuilder
					.newEntityContentProducer(getReference());
			sb.append(sep.getContent(getReference()));

			String text = StringUtils.escapeHtml(sb.toString(), false);
			TokenStream tokenStream = analyzer.tokenStream(
					SearchService.FIELD_CONTENTS, new StringReader(text));
			return hightlighter.getBestFragments(tokenStream, text, 5, " ... "); //$NON-NLS-1$
		}
		catch (IOException e)
		{
			return Messages.getString("SearchResultResponseImpl.11") + e.getMessage(); //$NON-NLS-1$
		}
	}

	public String getReference()
	{
		return (String) attributes.get("reference"); //$NON-NLS-1$
	}

	public TermFrequency getTerms() throws IOException
	{
		return null;
	}

	public void toXMLString(StringBuffer sb)
	{
		sb.append("<result"); //$NON-NLS-1$
		sb.append(" index=\"").append(getIndex()).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		sb.append(" score=\"").append(getScore()).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		sb.append(" sid=\"").append(StringUtils.xmlEscape(getId())).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		sb.append(" reference=\"").append(StringUtils.xmlEscape(getReference())).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		try
		{
			sb.append(" title=\"").append( //$NON-NLS-1$
					new String(Base64.encodeBase64(getTitle().getBytes("UTF-8")),"UTF-8")).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		catch (UnsupportedEncodingException e)
		{
			sb.append(" title=\"").append(StringUtils.xmlEscape(getTitle())).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		}
		sb.append(" tool=\"").append(StringUtils.xmlEscape(getTool())).append("\" "); //$NON-NLS-1$ //$NON-NLS-2$
		sb.append(" url=\"").append(StringUtils.xmlEscape(getUrl())).append("\" />"); //$NON-NLS-1$ //$NON-NLS-2$
	}

}
