/**********************************************************************************
 * $URL: https://source.sakaiproject.org/svn/site-manage/tags/sakai_2-4-0/site-manage-api/api/src/java/org/sakaiproject/sitemanage/api/SectionFieldManager.java $
 * $Id: SectionFieldManager.java 22875 2007-03-19 02:31:42Z daisyf@stanford.edu $
 ***********************************************************************************
 *
 * Copyright (c) 2007 The Sakai Foundation.
 * 
 * Licensed under the Educational Community License, Version 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 *      http://www.opensource.org/licenses/ecl1.php
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 **********************************************************************************/
package org.sakaiproject.sitemanage.api;

import java.util.List;


/**
 * The site management applications allow installations to configure a variable
 * number of "fields" to use when looking up a Section from the
 * CourseManagementService.  The SectionFieldManager provides method for
 * finding the number and type of fields to use in the UI, and for finding Section EIDs
 * from a number of Fields.
 * 
 * @author <a href="mailto:jholtzman@berkeley.edu">jholtzman@berkeley.edu</a>
 *
 */
public interface SectionFieldManager {
	
	/**
	 * Generates a Section EID from a List of fields.  This EID may be used to look up
	 * Sections from the CourseManagementService.  This may be implemented with
	 * simple text formatting, or an intermediary lookup may be needed to convert
	 * the human-understandable field values to a recognized EID.
	 * 
	 * @param fields
	 * @return
	 */
	public String getSectionEid(String academicSessionEid, List<SectionField> fields);

	/**
	 * Gets the List of SectionFields to use in the UI.
	 * 
	 * @return
	 */
	public List<SectionField> getRequiredFields();
}
